#!/usr/bin/env python3
# 
# notify_trello will connect to a Trello board and move cards around.
# Written by Jack Whitham, see https://www.jwhitham.org/2017/10/notifications.html
#
# To make best use of this script, use two Trello accounts, one for yourself (for manual use)
# and one for automated use. Create a board, and give the automated account access to it.
# Then log in to the automated account, and obtain the correct values for the following
# three constants:
#
# TRELLO_API_KEY is your 32 character developer API key. Get this from https://trello.com/app-key
# after logging in to Trello.
TRELLO_API_KEY = "fill in here"
# TRELLO_API_TOKEN is your 64 character secret. This is also from https://trello.com/app-key.
TRELLO_API_TOKEN = "fill in here"
# BOARD_ID is the name of one of your Trello boards. Find this within the URL of your board, e.g.
# https://trello.com/b/XyzAbcdE/qwerty  -->  board id XyzAbcdE
BOARD_ID = "fill in here"
#
#



import sys, getopt, subprocess, time, calendar
from trello import TrelloApi

UNITS = {
         "sec" : 1,
         "second" : 1,
         "min" : 60,
         "minute" : 60,
         "hour" : 60 * 60,
         "day" : 24 * 60 * 60,
         "week" : 24 * 60 * 60 * 7,
         "month" : 24 * 60 * 60 * 7 * 4,
         }
UNITS_IGNORE = ")(][}{,;.!"


def main():
   help_text = ("""
notify_trello will connect to a Trello board and move cards around.
Written by Jack Whitham, see https://www.jwhitham.org/2017/10/notifications.html

Usage:   
   notify_trello --card <card>
      move <card> to Done list

   notify_trello --card <card> -- <program> <args>
      move <card> to Running list, add <program> and <args>
      run <program> with <args>
      move <card> to Done list, add program output
   
   notify_trello --card <card> --to <list>
      move <card> to <list>, add <text> if provided

   notify_trello --auto-rotate <list1> --to <list2>
      examine cards in <list1>: if one has a interval in the
      description (e.g. "interval 30 days") and was not moved during
      that interval, move it to <list2>

If --add is given, then the card is created if it does not already exist.
If --message <text> appears, put <text> on the card.
If --verbose is given, report actions.

""")
   if len(sys.argv) <= 1:
      print (help_text)
      sys.exit(1)

   try:
      (option_value_list, args) = getopt.getopt(
         args = sys.argv[1:],
         shortopts = "avm:t:c:?hr",
         longopts = ["add", "verbose", "message=", "to=", "card=", "help", "auto-rotate="])
   except getopt.GetoptError as x:
      print (str(x))
      option_value_list = [("?", "")]

   card_name = None
   messages = ""
   target_list_name = "Done"
   add = False
   verbose = False
   auto_rotate_list_name = None

   for (option, value) in option_value_list:
      if option in ("a", "--add"):
         add = True
      elif option in ("v", "--verbose"):
         verbose = True
      elif option in ("m", "--message"):
         messages += value
      elif option in ("t", "--to"):
         target_list_name = value
      elif option in ("c", "--card"):
         card_name = value
      elif option in ("?", "h", "--help"):
         print (help_text)
         sys.exit(1)
      elif option in ("--auto-rotate"):
         auto_rotate_list_name = value

   if (card_name == None) and (auto_rotate_list_name == None):
      print ("You must specify the card name.")
      print (help_text)
      sys.exit(1)

   messages = messages.strip()

   if verbose: print ("API login...")
   trello_handle = TrelloApi(TRELLO_API_KEY)
   trello_handle.set_token(TRELLO_API_TOKEN)
   if verbose: print ("board download...")
   lists = trello_handle.boards.get_list(BOARD_ID, fields = "id,name")

   card = None
   target_list_id = None
   running_list_id = None
   auto_rotate_list_id = None
   card_count = 0
   for l in lists:
      list_id = l["id"]
      if card == None:
         if verbose: print ("card search " + list_id + "...")
         cards = trello_handle.lists.get_card(list_id, fields = "id,name")
         for c in cards:
            if c["name"] == card_name:
               card = c
               break
            card_count += 1

      if l["name"] == target_list_name:
         target_list_id = list_id
      if l["name"] == "Running":
         running_list_id = list_id
      if l["name"] == auto_rotate_list_name:
         auto_rotate_list_id = list_id


   if target_list_id == None:
      print ("Unable to find a list named '%s' - "
             "found %u other lists" % (
               target_list_name, len(lists)))
      sys.exit(1)

   if running_list_id == None:
      print ("Unable to find a list named '%s' - "
             "found %u other lists" % (
               "Running", len(lists)))
      sys.exit(1)

   if (auto_rotate_list_id == None) and (auto_rotate_list_name != None):
      print ("Unable to find a list named '%s' - "
             "found %u other lists" % (
               auto_rotate_list_name, len(lists)))
      sys.exit(1)

   if auto_rotate_list_name != None:
      cards = trello_handle.lists.get_card(auto_rotate_list_id, fields = "id,name,desc,dateLastActivity")
      for c in cards:
         card_name = c["name"]
         desc = card_name + " " + c["desc"]
         desc_fields = desc.lower().split()
         value = None
         for i in range(max(0, len(desc_fields) - 2)):
            text = desc_fields[i].lstrip(UNITS_IGNORE)
            if text == "interval":
               value = desc_fields[i + 1]
               unit = desc_fields[i + 2].rstrip(UNITS_IGNORE + "s")
               try:
                  value = int(value)
               except:
                  value = None
               
               if value <= 0:
                  value = None
               if UNITS.get(unit, None) == None:
                  value = None
          
               if value == None:
                  print ("Card '%s' on list '%s' has an invalid interval '%s %s'" %
                           (card_name, auto_rotate_list_name, desc_fields[i + 1], desc_fields[i + 2]))
                  sys.exit(1)
               
               value *= UNITS[unit]
               break
      
         if value != None:
            print ("card '%s' on list '%s' has interval %d seconds" % (
                  card_name, auto_rotate_list_name, value))
            iso8601_date = c["dateLastActivity"]
            last_moved = calendar.timegm(time.strptime(iso8601_date, "%Y-%m-%dT%H:%M:%S.%fZ"))
            age = int(time.time() - last_moved)
            if age >= value:
               trello_handle.cards.update(c["id"], idList = target_list_id)
               print ("  last moved %d seconds ago: moved to list '%s'" % (age, target_list_name))
            else:
               print ("  last moved %d seconds ago: no move" % (age))

      sys.exit(0)

   if card == None:
      if add:
         if verbose: print ("create card...")
         card = trello_handle.cards.new(card_name, idList = running_list_id, desc = messages)
      else:
         print ("Unable to find a card named '%s' in any of the "
                "%u lists - found %u other cards" % (
                  card_name, len(lists), card_count))
         sys.exit(1)

   rc = 0
   if len(args) != 0:
      if len(messages) != 0:
         messages += "\n"
      messages += "command: " + " ".join(args)
      if verbose: print ("prepare to run...")
      trello_handle.cards.update(card["id"], desc = messages, idList = running_list_id)
      if verbose: print ("run...")
      try:
         rc = subprocess.call(args)
      except KeyboardInterrupt:
         rc = -1
      messages += "\nreturn: %d" % rc

   if verbose: print ("move card...")
   trello_handle.cards.update(card["id"], desc = messages, idList = target_list_id)
   if verbose: print ("finished")
   sys.exit(rc)

if __name__ == "__main__":
   main()

